@@ -2,6 +2,7 @@ package App::Ack;
 
 use warnings;
 use strict;
+use File::Basename;
 
 =head1 NAME
 
@@ -9,11 +10,11 @@ App::Ack - A container for functions for the ack program
 
 =head1 VERSION
 
-Version 1.25_03
+Version 1.27_01
 
 =cut
 
-our $VERSION = '1.25_03';
+our $VERSION = '1.27_01';
 
 =head1 SYNOPSIS
 
@@ -58,29 +59,44 @@ sub skipdir_filter {
 our %types;
 our %mappings = (
     asm         => [qw( s S )],
-    binary      => q{Binary files as defined by Perl's -B operator},
+    binary      => q{Binary files, as defined by Perl's -B op (default: off)},
     cc          => [qw( c h )],
     css         => [qw( css )],
+    elisp       => [qw( el )],
+    haskell     => [qw( hs lhs )],
+    html        => [qw( htm html shtml )],
+    lisp        => [qw( lisp )],
     js          => [qw( js )],
+    ocaml       => [qw( ml mli )],
     parrot      => [qw( pir pasm pmc ops pod pg tg )],
     perl        => [qw( pl pm pod tt ttml t )],
     php         => [qw( php phpt htm html )],
     python      => [qw( py )],
-    ruby        => [qw( rb )],
+    ruby        => [qw( rb rhtml rjs )],
+    scheme      => [qw( scm )],
     shell       => [qw( sh bash csh ksh zsh )],
     sql         => [qw( sql ctl )],
+    tt          => [qw( tt tt2 )],
+    vim         => [qw( vim )],
     yaml        => [qw( yaml yml )],
+    -ignore     => [qw( a o so swp core )],
 );
 
+our @suffixes;
+
 sub _init_types {
+    my %suffixes;
     while ( my ($type,$exts) = each %mappings ) {
         if ( ref $exts ) {
             for my $ext ( @$exts ) {
                 push( @{$types{$ext}}, $type );
+                ++$suffixes{"\\.$ext"};
             }
         }
     }
 
+    @suffixes = keys %suffixes;
+
     return;
 }
 
@@ -97,9 +113,18 @@ sub filetypes {
 
     _init_types() unless keys %types;
 
+    return '-ignore' if $filename =~ /~$/;
+
+    # Pass our $filename in lowercase so we match lowercase filenames
+    my ($filebase,$dirs,$suffix) = File::Basename::fileparse( lc $filename, @suffixes );
+
+    return '-ignore' if $filebase =~ /^#.+#$/;
+    return '-ignore' if $filebase =~ /^core\.\d+$/;
+
     # If there's an extension, look it up
-    if ( $filename =~ /\.([^.]+)$/ ) {
-        my $ref = $types{lc $1};
+    if ( $suffix ) {
+        $suffix =~ s/^\.//; # Drop the period that File::Basename needs
+        my $ref = $types{lc $suffix};
         return @$ref if $ref;
     }
 
@@ -150,30 +175,30 @@ Dumps the help page to the user.
 =cut
 
 sub show_help {
-    my @lines = <DATA>;
+    my $help = join "", <DATA>;
 
-    for ( @lines ) {
-        s/(\w+)(\s+)LIST/$1.$2._expand_list($1)/esmx;
-        s/IGNORE_DIRS/_ignore_dirs_str()/esmx;
+    my @langlines;
+    for my $lang ( sort keys %mappings ) {
+        next if $lang =~ /^-/; # Stuff to not show
+        my $ext_list = $mappings{$lang};
+
+        if ( ref $ext_list ) {
+            my @exts = map { ".$_" } @$ext_list;
+
+            $ext_list = _listify( @exts );
+        }
+        push( @langlines, sprintf( '    --[no]%-9.9s %s', $lang, $ext_list ) );
     }
-    print @lines;
+    my $langlines = join( "\n", @langlines );
+
+    $help =~ s/LIST/$langlines/smx;
+    $help =~ s/IGNORE_DIRS/_ignore_dirs_str()/esmx;
+
+    print $help;
 
     return;
 }
 
-sub _expand_list {
-    my $lang = shift;
-
-    my $ext_list = $mappings{$lang};
-
-    if ( not ref $ext_list ) {
-        return $ext_list;
-    }
-    my @files = map { ".$_" } @{$mappings{$lang}};
-
-    return _listify( @files );
-}
-
 sub _listify {
     my @whats = @_;
 
@@ -282,19 +307,7 @@ File inclusion/exclusion:
     -n              No descending into subdirectories
     -a, --all       All files, regardless of extension (but still skips
                     IGNORE_DIRS dirs)
-    --[no]asm       LIST
-    --[no]cc        LIST
-    --[no]js        LIST
-    --[no]parrot    LIST
-    --[no]perl      LIST
-    --[no]php       LIST
-    --[no]python    LIST
-    --[no]ruby      LIST
-    --[no]shell     LIST
-    --[no]sql       LIST
-    --[no]yaml      LIST
-    --[no]binary    LIST
-                    (default: no binary)
+LIST
 
 Miscellaneous:
     --help          this help
